@@ -1,10 +1,9 @@
 use crate::{
     db::{Database, DatabaseCommit, DatabaseRef, RefDBWrapper},
-    error::ExitReason,
     evm_impl::{EVMImpl, Transact},
     subroutine::State,
     BerlinSpec, ByzantineSpec, Env, Inspector, IstanbulSpec, LatestSpec, LondonSpec, NoOpInspector,
-    Spec, SpecId, TransactOut,
+    Return, Spec, SpecId, TransactOut,
 };
 use alloc::boxed::Box;
 use revm_precompiles::Precompiles;
@@ -33,9 +32,15 @@ pub fn new<DB>() -> EVM<DB> {
     EVM::new()
 }
 
+impl<DB> Default for EVM<DB> {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
 impl<DB: Database + DatabaseCommit> EVM<DB> {
     /// Execute transaction and apply result to database
-    pub fn transact_commit(&mut self) -> (ExitReason, TransactOut, u64) {
+    pub fn transact_commit(&mut self) -> (Return, TransactOut, u64) {
         let (exit, out, gas, state) = self.transact();
         self.db.as_mut().unwrap().commit(state);
         (exit, out, gas)
@@ -44,7 +49,7 @@ impl<DB: Database + DatabaseCommit> EVM<DB> {
     pub fn inspect_commit<INSP: Inspector>(
         &mut self,
         inspector: INSP,
-    ) -> (ExitReason, TransactOut, u64) {
+    ) -> (Return, TransactOut, u64) {
         let (exit, out, gas, state) = self.inspect(inspector);
         self.db.as_mut().unwrap().commit(state);
         (exit, out, gas)
@@ -53,11 +58,10 @@ impl<DB: Database + DatabaseCommit> EVM<DB> {
 
 impl<DB: Database> EVM<DB> {
     /// Execute transaction without writing to DB, return change state.
-    pub fn transact(&mut self) -> (ExitReason, TransactOut, u64, State) {
+    pub fn transact(&mut self) -> (Return, TransactOut, u64, State) {
         if let Some(db) = self.db.as_mut() {
             let mut noop = NoOpInspector {};
-            let out =
-                evm_inner::<DB, false>(self.env.cfg.spec_id, &self.env, db, &mut noop).transact();
+            let out = evm_inner::<DB, false>(&self.env, db, &mut noop).transact();
             out
         } else {
             panic!("Database needs to be set");
@@ -68,9 +72,9 @@ impl<DB: Database> EVM<DB> {
     pub fn inspect<INSP: Inspector>(
         &mut self,
         mut inspector: INSP,
-    ) -> (ExitReason, TransactOut, u64, State) {
+    ) -> (Return, TransactOut, u64, State) {
         if let Some(db) = self.db.as_mut() {
-            evm_inner::<DB, true>(self.env.cfg.spec_id, &self.env, db, &mut inspector).transact()
+            evm_inner::<DB, true>(&self.env, db, &mut inspector).transact()
         } else {
             panic!("Database needs to be set");
         }
@@ -79,14 +83,12 @@ impl<DB: Database> EVM<DB> {
 
 impl<DB: DatabaseRef> EVM<DB> {
     /// Execute transaction without writing to DB, return change state.
-    pub fn transact_ref(&self) -> (ExitReason, TransactOut, u64, State) {
+    pub fn transact_ref(&self) -> (Return, TransactOut, u64, State) {
         if let Some(db) = self.db.as_ref() {
             let mut noop = NoOpInspector {};
             let mut db = RefDBWrapper::new(db);
             let db = &mut db;
-            let out =
-                evm_inner::<RefDBWrapper, false>(self.env.cfg.spec_id, &self.env, db, &mut noop)
-                    .transact();
+            let out = evm_inner::<RefDBWrapper, false>(&self.env, db, &mut noop).transact();
             out
         } else {
             panic!("Database needs to be set");
@@ -97,17 +99,11 @@ impl<DB: DatabaseRef> EVM<DB> {
     pub fn inspect_ref<INSP: Inspector>(
         &self,
         mut inspector: INSP,
-    ) -> (ExitReason, TransactOut, u64, State) {
+    ) -> (Return, TransactOut, u64, State) {
         if let Some(db) = self.db.as_ref() {
             let mut db = RefDBWrapper::new(db);
             let db = &mut db;
-            let out = evm_inner::<RefDBWrapper, true>(
-                self.env.cfg.spec_id,
-                &self.env,
-                db,
-                &mut inspector,
-            )
-            .transact();
+            let out = evm_inner::<RefDBWrapper, true>(&self.env, db, &mut inspector).transact();
             out
         } else {
             panic!("Database needs to be set");
@@ -137,7 +133,7 @@ impl<DB> EVM<DB> {
 }
 
 macro_rules! create_evm {
-    ($spec:tt,$db:ident,$env:ident,$inspector:ident) => {
+    ($spec:ident, $db:ident,$env:ident,$inspector:ident) => {
         Box::new(EVMImpl::<'a, $spec, DB, INSPECT>::new(
             $db,
             $env,
@@ -147,13 +143,12 @@ macro_rules! create_evm {
     };
 }
 
-fn evm_inner<'a, DB: Database, const INSPECT: bool>(
-    specid: SpecId,
+pub fn evm_inner<'a, DB: Database, const INSPECT: bool>(
     env: &'a Env,
     db: &'a mut DB,
     insp: &'a mut dyn Inspector,
 ) -> Box<dyn Transact + 'a> {
-    match specid {
+    match env.cfg.spec_id {
         SpecId::LATEST => create_evm!(LatestSpec, db, env, insp),
         SpecId::LONDON => create_evm!(LondonSpec, db, env, insp),
         SpecId::BERLIN => create_evm!(BerlinSpec, db, env, insp),
