@@ -1,13 +1,13 @@
 use crate::{
     db::Database,
-    error::{ExitError, ExitReason, ExitSucceed},
+    instructions::gas,
     machine,
     machine::{Contract, Gas, Machine},
     models::SelfDestructResult,
-    opcode::gas,
+    return_ok,
     spec::{Spec, SpecId::*},
     subroutine::{Account, State, SubRoutine},
-    util, CallContext, CreateScheme, Env, ExitRevert, Inspector, Log, TransactOut, TransactTo,
+    util, CallContext, CreateScheme, Env, Inspector, Log, Return, TransactOut, TransactTo,
     Transfer, KECCAK_EMPTY,
 };
 use alloc::vec::Vec;
@@ -29,25 +29,25 @@ pub struct EVMImpl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> {
 
 pub trait Transact {
     /// Do transaction.
-    /// Return ExitReason, Output for call or Address if we are creating contract, gas spend, State that needs to be applied.
-    fn transact(&mut self) -> (ExitReason, TransactOut, u64, State);
+    /// Return Return, Output for call or Address if we are creating contract, gas spend, State that needs to be applied.
+    fn transact(&mut self) -> (Return, TransactOut, u64, State);
 }
 
 impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Transact
     for EVMImpl<'a, GSPEC, DB, INSPECT>
 {
-    fn transact(&mut self) -> (ExitReason, TransactOut, u64, State) {
+    fn transact(&mut self) -> (Return, TransactOut, u64, State) {
         let caller = self.env.tx.caller;
         let value = self.env.tx.value;
         let data = self.env.tx.data.clone();
         let gas_limit = self.env.tx.gas_limit;
-        let exit_error = |reason: ExitReason| (reason, TransactOut::None, 0, State::new());
+        let exit = |reason: Return| (reason, TransactOut::None, 0, State::new());
 
         if GSPEC::enabled(LONDON) {
             if let Some(priority_fee) = self.env.tx.gas_priority_fee {
                 if priority_fee > self.env.tx.gas_price {
                     // or gas_max_fee for eip1559
-                    return exit_error(ExitError::GasMaxFeeGreaterThanPriorityFee.into());
+                    return exit(Return::GasMaxFeeGreaterThanPriorityFee);
                 }
             }
             let effective_gas_price = self.env.effective_gas_price();
@@ -57,19 +57,19 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Transact
             // TODO maybe do this checks when creating evm. We already have all data there
             // or should be move effective_gas_price inside transact fn
             if effective_gas_price < basefee {
-                return exit_error(ExitError::GasPriceLessThenBasefee.into());
+                return exit(Return::GasPriceLessThenBasefee);
             }
             // check if priority fee is lower then max fee
         }
         // unusual to be found here, but check if gas_limit is more then block_gas_limit
         if U256::from(gas_limit) > self.env.block.gas_limit {
-            return exit_error(ExitError::CallerGasLimitMoreThenBlock.into());
+            return exit(Return::CallerGasLimitMoreThenBlock);
         }
 
         let mut gas = Gas::new(gas_limit);
-        // record initial gas cost.
+        // record initial gas cost. if not using gas metering init will return 0
         if !gas.record_cost(self.initialization::<GSPEC>()) {
-            return exit_error(ExitError::OutOfGas.into());
+            return exit(Return::OutOfGas);
         }
 
         // load acc
@@ -79,7 +79,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Transact
         // This EIP is introduced after london but there was no colision in past
         // so we can leave it enabled always
         if self.subroutine.account(caller).info.code_hash != KECCAK_EMPTY {
-            return exit_error(ExitError::RejectCallerWithCode.into());
+            return exit(Return::RejectCallerWithCode);
         }
 
         // substract gas_limit*gas_price from current account.
@@ -87,21 +87,23 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Transact
             U256::from(gas_limit).checked_mul(self.env.effective_gas_price())
         {
             if !self.subroutine.balance_sub(caller, payment_value) {
-                return exit_error(ExitError::LackOfFundForGasLimit.into());
+                return exit(Return::LackOfFundForGasLimit);
             }
         } else {
-            return exit_error(ExitError::OverflowPayment.into());
+            return exit(Return::OverflowPayment);
         }
 
         // check if we have enought balance for value transfer.
         let difference = self.env.tx.gas_price - self.env.effective_gas_price();
         if difference + value > self.subroutine.account(caller).info.balance {
-            return exit_error(ExitError::OutOfFund.into());
+            return exit(Return::OutOfFund);
         }
 
         // record all as cost;
         let gas_limit = gas.remaining();
-        gas.record_cost(gas_limit);
+        if crate::USE_GAS {
+            gas.record_cost(gas_limit);
+        }
 
         // call inner handling of call/create
         let (exit_reason, ret_gas, out) = match self.env.tx.transact_to {
@@ -132,7 +134,9 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Transact
             }
         };
 
-        gas.reimburse_unspend(&exit_reason, ret_gas);
+        if crate::USE_GAS {
+            gas.reimburse_unspend(&exit_reason, ret_gas);
+        }
         match self.finalize::<GSPEC>(caller, &gas) {
             Err(e) => (e, out, gas.spend(), Map::new()),
             Ok(state) => (exit_reason, out, gas.spend(), state),
@@ -149,7 +153,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
     ) -> Self {
         let mut precompile_acc = Map::new();
         for (add, _) in precompiles.as_slice() {
-            precompile_acc.insert(add.clone(), db.basic(add.clone()));
+            precompile_acc.insert(*add, db.basic(*add));
         }
         Self {
             db,
@@ -165,26 +169,31 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         &mut self,
         caller: H160,
         gas: &Gas,
-    ) -> Result<Map<H160, Account>, ExitReason> {
-        let effective_gas_price = self.env.effective_gas_price();
+    ) -> Result<Map<H160, Account>, Return> {
         let coinbase = self.env.block.coinbase;
-        let basefee = self.env.block.basefee;
-        let max_refund_quotient = if SPEC::enabled(LONDON) { 5 } else { 2 }; // EIP-3529: Reduction in refunds
-        let gas_refunded = min(gas.refunded() as u64, gas.spend() / max_refund_quotient);
-        self.subroutine.balance_add(
-            caller,
-            effective_gas_price * (gas.remaining() + gas_refunded),
-        );
-        let coinbase_gas_price = if SPEC::enabled(LONDON) {
-            effective_gas_price.saturating_sub(basefee)
+        if crate::USE_GAS {
+            let effective_gas_price = self.env.effective_gas_price();
+            let basefee = self.env.block.basefee;
+            let max_refund_quotient = if SPEC::enabled(LONDON) { 5 } else { 2 }; // EIP-3529: Reduction in refunds
+            let gas_refunded = min(gas.refunded() as u64, gas.spend() / max_refund_quotient);
+            self.subroutine.balance_add(
+                caller,
+                effective_gas_price * (gas.remaining() + gas_refunded),
+            );
+            let coinbase_gas_price = if SPEC::enabled(LONDON) {
+                effective_gas_price.saturating_sub(basefee)
+            } else {
+                effective_gas_price
+            };
+
+            self.subroutine.load_account(coinbase, self.db);
+            self.subroutine
+                .balance_add(coinbase, coinbase_gas_price * (gas.spend() - gas_refunded));
         } else {
-            effective_gas_price
-        };
-        //if coinbase_gas_price != U256::zero() {
-        self.subroutine.load_account(coinbase, self.db);
-        self.subroutine
-            .balance_add(coinbase, coinbase_gas_price * (gas.spend() - gas_refunded));
-        //}
+            // touch coinbase
+            self.subroutine.load_account(coinbase, self.db);
+            self.subroutine.balance_add(coinbase, U256::zero());
+        }
         Ok(self.subroutine.finalize())
     }
 
@@ -202,49 +211,53 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         let access_list = self.env.tx.access_list.clone();
         for (ward_acc, _) in self.precompiles.as_slice() {
             //TODO trace load precompiles?
-            self.subroutine.load_account(ward_acc.clone(), self.db);
+            self.subroutine.load_account(*ward_acc, self.db);
         }
 
-        let zero_data_len = input.iter().filter(|v| **v == 0).count() as u64;
-        let non_zero_data_len = (input.len() as u64 - zero_data_len) as u64;
-        let (accessed_accounts, accessed_slots) = {
-            if SPEC::enabled(BERLIN) {
-                let mut accessed_slots = 0 as u64;
-                let accessed_accounts = access_list.len() as u64;
+        if crate::USE_GAS {
+            let zero_data_len = input.iter().filter(|v| **v == 0).count() as u64;
+            let non_zero_data_len = (input.len() as u64 - zero_data_len) as u64;
+            let (accessed_accounts, accessed_slots) = {
+                if SPEC::enabled(BERLIN) {
+                    let mut accessed_slots = 0_u64;
+                    let accessed_accounts = access_list.len() as u64;
 
-                for (address, slots) in access_list {
-                    //TODO trace load access_list?
-                    self.subroutine.load_account(address, self.db);
-                    accessed_slots += slots.len() as u64;
-                    for slot in slots {
-                        self.subroutine.sload(address, slot, self.db);
+                    for (address, slots) in access_list {
+                        //TODO trace load access_list?
+                        self.subroutine.load_account(address, self.db);
+                        accessed_slots += slots.len() as u64;
+                        for slot in slots {
+                            self.subroutine.sload(address, slot, self.db);
+                        }
                     }
+                    (accessed_accounts, accessed_slots)
+                } else {
+                    (0, 0)
                 }
-                (accessed_accounts, accessed_slots)
-            } else {
-                (0, 0)
-            }
-        };
+            };
 
-        let transact = if is_create {
-            if SPEC::enabled(HOMESTEAD) {
-                // EIP-2: Homestead Hard-fork Changes
-                53000
+            let transact = if is_create {
+                if SPEC::enabled(HOMESTEAD) {
+                    // EIP-2: Homestead Hard-fork Changes
+                    53000
+                } else {
+                    21000
+                }
             } else {
                 21000
-            }
+            };
+
+            // EIP-2028: Transaction data gas cost reduction
+            let gas_transaction_non_zero_data = if SPEC::enabled(ISTANBUL) { 16 } else { 68 };
+
+            transact
+                + zero_data_len * gas::TRANSACTION_ZERO_DATA
+                + non_zero_data_len * gas_transaction_non_zero_data
+                + accessed_accounts * gas::ACCESS_LIST_ADDRESS
+                + accessed_slots * gas::ACCESS_LIST_STORAGE_KEY
         } else {
-            21000
-        };
-
-        // EIP-2028: Transaction data gas cost reduction
-        let gas_transaction_non_zero_data = if SPEC::enabled(ISTANBUL) { 16 } else { 68 };
-
-        transact
-            + zero_data_len * gas::TRANSACTION_ZERO_DATA
-            + non_zero_data_len * gas_transaction_non_zero_data
-            + accessed_accounts * gas::ACCESS_LIST_ADDRESS
-            + accessed_slots * gas::ACCESS_LIST_STORAGE_KEY
+            0
+        }
     }
 
     fn create_inner<SPEC: Spec>(
@@ -254,17 +267,17 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         value: U256,
         init_code: Bytes,
         gas_limit: u64,
-    ) -> (ExitReason, Option<H160>, Gas, Bytes) {
+    ) -> (Return, Option<H160>, Gas, Bytes) {
         let gas = Gas::new(gas_limit);
         self.load_account(caller);
 
         // check depth of calls
         if self.subroutine.depth() > machine::CALL_STACK_LIMIT {
-            return (ExitRevert::CallTooDeep.into(), None, gas, Bytes::new());
+            return (Return::CallTooDeep, None, gas, Bytes::new());
         }
         // check balance of caller and value. Do this before increasing nonce
         if self.balance(caller).0 < value {
-            return (ExitRevert::OutOfFund.into(), None, gas, Bytes::new());
+            return (Return::OutOfFund, None, gas, Bytes::new());
         }
 
         // inc nonce of caller
@@ -290,7 +303,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
             self.db,
         ) {
             self.subroutine.checkpoint_revert(checkpoint);
-            return (ExitError::CreateCollision.into(), ret, gas, Bytes::new());
+            return (Return::CreateCollision, ret, gas, Bytes::new());
         }
 
         // transfer value to contract address
@@ -299,7 +312,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
             .transfer(caller, created_address, value, self.db)
         {
             self.subroutine.checkpoint_revert(checkpoint);
-            return (e.into(), ret, gas, Bytes::new());
+            return (e, ret, gas, Bytes::new());
         }
         // inc nonce of contract
         if SPEC::enabled(ISTANBUL) {
@@ -311,7 +324,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         let exit_reason = machine.run::<Self, SPEC>(self);
         // handler error if present on execution\
         match exit_reason {
-            ExitReason::Succeed(_) => {
+            return_ok!() => {
                 let b = Bytes::new();
                 // if ok, check contract creation limit and calculate gas deduction on output len.
                 let code = machine.return_value();
@@ -319,26 +332,27 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
                 // EIP-3541: Reject new contract code starting with the 0xEF byte
                 if SPEC::enabled(LONDON) && !code.is_empty() && code.get(0) == Some(&0xEF) {
                     self.subroutine.checkpoint_revert(checkpoint);
-                    return (ExitError::CreateContractWithEF.into(), ret, machine.gas, b);
+                    return (Return::CreateContractWithEF, ret, machine.gas, b);
                 }
 
                 // EIP-170: Contract code size limit
                 if SPEC::enabled(SPURIOUS_DRAGON) && code.len() > 0x6000 {
                     self.subroutine.checkpoint_revert(checkpoint);
-                    return (ExitError::CreateContractLimit.into(), ret, machine.gas, b);
+                    return (Return::CreateContractLimit, ret, machine.gas, b);
                 }
-                let gas_for_code = code.len() as u64 * crate::opcode::gas::CODEDEPOSIT;
-                // record code deposit gas cost and check if we are out of gas.
-                if !machine.gas.record_cost(gas_for_code) {
-                    self.subroutine.checkpoint_revert(checkpoint);
-                    (ExitError::OutOfGas.into(), ret, machine.gas, b)
-                } else {
-                    // if we have enought gas
-                    self.subroutine.checkpoint_commit();
-                    let code_hash = H256::from_slice(Keccak256::digest(&code).as_slice());
-                    self.subroutine.set_code(created_address, code, code_hash);
-                    (ExitSucceed::Returned.into(), ret, machine.gas, b)
+                if crate::USE_GAS {
+                    let gas_for_code = code.len() as u64 * crate::instructions::gas::CODEDEPOSIT;
+                    // record code deposit gas cost and check if we are out of gas.
+                    if !machine.gas.record_cost(gas_for_code) {
+                        self.subroutine.checkpoint_revert(checkpoint);
+                        return (Return::OutOfGas, ret, machine.gas, b);
+                    }
                 }
+                // if we have enought gas
+                self.subroutine.checkpoint_commit();
+                let code_hash = H256::from_slice(Keccak256::digest(&code).as_slice());
+                self.subroutine.set_code(created_address, code, code_hash);
+                (Return::Continue, ret, machine.gas, b)
             }
             _ => {
                 self.subroutine.checkpoint_revert(checkpoint);
@@ -355,20 +369,20 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         input: Bytes,
         gas_limit: u64,
         context: CallContext,
-    ) -> (ExitReason, Gas, Bytes) {
+    ) -> (Return, Gas, Bytes) {
         let mut gas = Gas::new(gas_limit);
         // Load account and get code. Account is now hot.
         let (code, _) = self.code(code_address);
 
         // check depth
         if self.subroutine.depth() > machine::CALL_STACK_LIMIT {
-            return (ExitRevert::CallTooDeep.into(), gas, Bytes::new());
+            return (Return::CallTooDeep, gas, Bytes::new());
         }
 
         // Create subroutine checkpoint
         let checkpoint = self.subroutine.create_checkpoint();
         // touch address. For "EIP-158 State Clear" this will erase empty accounts.
-        if transfer.value == U256::zero() {
+        if transfer.value.is_zero() {
             self.load_account(context.address);
             self.subroutine.balance_add(context.address, U256::zero()); // touch the acc
         }
@@ -380,7 +394,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         {
             Err(e) => {
                 self.subroutine.checkpoint_revert(checkpoint);
-                return (e.into(), gas, Bytes::new());
+                return (e, gas, Bytes::new());
             }
             Ok((source_is_cold, target_is_cold)) => {
                 if INSPECT && source_is_cold {
@@ -400,7 +414,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
             };
             match out {
                 Ok(PrecompileOutput { output, cost, logs }) => {
-                    if gas.record_cost(cost) {
+                    if !crate::USE_GAS || gas.record_cost(cost) {
                         logs.into_iter().for_each(|l| {
                             self.subroutine.log(Log {
                                 address: l.address,
@@ -409,15 +423,15 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
                             })
                         });
                         self.subroutine.checkpoint_commit();
-                        (ExitSucceed::Returned.into(), gas, Bytes::from(output))
+                        (Return::Continue, gas, Bytes::from(output))
                     } else {
                         self.subroutine.checkpoint_revert(checkpoint);
-                        (ExitError::OutOfGas.into(), gas, Bytes::new())
+                        (Return::OutOfGas, gas, Bytes::new())
                     }
                 }
-                Err(e) => {
+                Err(_e) => {
                     self.subroutine.checkpoint_revert(checkpoint); //TODO check if we are discarding or reverting
-                    (ExitError::Precompile(e).into(), gas, Bytes::new())
+                    (Return::Precompile, gas, Bytes::new())
                 }
             }
         } else {
@@ -425,7 +439,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
             let contract = Contract::new_with_context(input, code, &context);
             let mut machine = Machine::new::<SPEC>(contract, gas_limit, self.subroutine.depth());
             let exit_reason = machine.run::<Self, SPEC>(self);
-            if matches!(exit_reason, ExitReason::Succeed(_)) {
+            if matches!(exit_reason, return_ok!()) {
                 self.subroutine.checkpoint_commit();
             } else {
                 self.subroutine.checkpoint_revert(checkpoint);
@@ -442,7 +456,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
     const INSPECT: bool = INSPECT;
 
     fn env(&self) -> &Env {
-        &self.env
+        self.env
     }
 
     fn inspect(&mut self) -> &mut dyn Inspector {
@@ -491,12 +505,12 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
         )
     }
 
-    fn sload(&mut self, address: H160, index: H256) -> (H256, bool) {
+    fn sload(&mut self, address: H160, index: U256) -> (U256, bool) {
         // account is allways hot. reference on that statement https://eips.ethereum.org/EIPS/eip-2929 see `Note 2:`
         self.subroutine.sload(address, index, self.db)
     }
 
-    fn sstore(&mut self, address: H160, index: H256, value: H256) -> (H256, H256, H256, bool) {
+    fn sstore(&mut self, address: H160, index: U256, value: U256) -> (U256, U256, U256, bool) {
         self.subroutine.sstore(address, index, value, self.db)
     }
 
@@ -524,7 +538,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
         value: U256,
         init_code: Bytes,
         gas: u64,
-    ) -> (ExitReason, Option<H160>, Gas, Bytes) {
+    ) -> (Return, Option<H160>, Gas, Bytes) {
         self.create_inner::<SPEC>(caller, scheme, value, init_code, gas)
     }
 
@@ -535,7 +549,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
         input: Bytes,
         gas: u64,
         context: CallContext,
-    ) -> (ExitReason, Gas, Bytes) {
+    ) -> (Return, Gas, Bytes) {
         self.call_inner::<SPEC>(code_address, transfer, input, gas, context)
     }
 }
@@ -559,9 +573,9 @@ pub trait Handler {
     /// Get code hash of address.
     fn code_hash(&mut self, address: H160) -> (H256, bool);
     /// Get storage value of address at index.
-    fn sload(&mut self, address: H160, index: H256) -> (H256, bool);
+    fn sload(&mut self, address: H160, index: U256) -> (U256, bool);
     /// Set storage value of address at index. Return if slot is cold/hot access.
-    fn sstore(&mut self, address: H160, index: H256, value: H256) -> (H256, H256, H256, bool);
+    fn sstore(&mut self, address: H160, index: U256, value: U256) -> (U256, U256, U256, bool);
     /// Create a log owned by address with given topics and data.
     fn log(&mut self, address: H160, topics: Vec<H256>, data: Bytes);
     /// Mark an address to be deleted, with funds transferred to target.
@@ -574,7 +588,7 @@ pub trait Handler {
         value: U256,
         init_code: Bytes,
         gas: u64,
-    ) -> (ExitReason, Option<H160>, Gas, Bytes);
+    ) -> (Return, Option<H160>, Gas, Bytes);
 
     /// Invoke a call operation.
     fn call<SPEC: Spec>(
@@ -584,5 +598,5 @@ pub trait Handler {
         input: Bytes,
         gas: u64,
         context: CallContext,
-    ) -> (ExitReason, Gas, Bytes);
+    ) -> (Return, Gas, Bytes);
 }
