@@ -1,7 +1,7 @@
 use bytes::Bytes;
 use primitive_types::{H160, H256, U256};
 
-use crate::{opcode::Control, CallContext, CreateScheme, ExitReason, Machine, Transfer};
+use crate::{CallContext, CreateScheme, Machine, Return, Transfer};
 use auto_impl::auto_impl;
 
 #[auto_impl(&mut, Box)]
@@ -9,19 +9,19 @@ pub trait Inspector {
     // get opcode by calling `machine.contract.opcode(machine.program_counter())`.
     // all other information can be obtained from machine.
     fn step(&mut self, machine: &mut Machine);
-    fn eval(&mut self, eval: &mut Control, machine: &mut Machine);
+    fn eval(&mut self, eval: Return, machine: &mut Machine);
 
     fn load_account(&mut self, address: &H160);
 
-    fn sload(&mut self, address: &H160, slot: &H256, value: &H256, is_cold: bool);
+    fn sload(&mut self, address: &H160, slot: &U256, value: &U256, is_cold: bool);
 
     fn sstore(
         &mut self,
         address: H160,
-        slot: H256,
-        new_value: H256,
-        old_value: H256,
-        original_value: H256,
+        slot: U256,
+        new_value: U256,
+        old_value: U256,
+        original_value: U256,
         is_cold: bool,
     );
 
@@ -35,7 +35,7 @@ pub trait Inspector {
         is_static: bool,
     );
 
-    fn call_return(&mut self, exit: ExitReason);
+    fn call_return(&mut self, exit: Return);
 
     fn create(
         &mut self,
@@ -57,19 +57,19 @@ pub struct NoOpInspector();
 impl Inspector for NoOpInspector {
     fn step(&mut self, _machine: &mut Machine) {}
 
-    fn eval(&mut self, _eval: &mut Control, _machine: &mut Machine) {}
+    fn eval(&mut self, _eval: Return, _machine: &mut Machine) {}
 
     fn load_account(&mut self, _address: &H160) {}
 
-    fn sload(&mut self, _address: &H160, _slot: &H256, _value: &H256, _is_cold: bool) {}
+    fn sload(&mut self, _address: &H160, _slot: &U256, _value: &U256, _is_cold: bool) {}
 
     fn sstore(
         &mut self,
         _address: H160,
-        _slot: H256,
-        _new_value: H256,
-        _old_value: H256,
-        _original_value: H256,
+        _slot: U256,
+        _new_value: U256,
+        _old_value: U256,
+        _original_value: U256,
         _is_cold: bool,
     ) {
     }
@@ -85,7 +85,7 @@ impl Inspector for NoOpInspector {
     ) {
     }
 
-    fn call_return(&mut self, _exit: ExitReason) {}
+    fn call_return(&mut self, _exit: Return) {}
 
     fn create(
         &mut self,
