@@ -1,14 +1,16 @@
-use crate::{alloc::vec::Vec, CallContext, ExitError, ExitReason, ExitSucceed};
+use crate::{alloc::vec::Vec, CallContext};
 use bytes::Bytes;
 use primitive_types::{H160, U256};
 
-use crate::opcode::OpCode;
+use crate::instructions::opcode::{self, OpCode};
 
 pub struct Contract {
     /// Contracts data
     pub input: Bytes,
     /// Contract code
     pub code: Bytes,
+    /// code size of original code. Note that current code is extended with push padding and STOP at end
+    pub code_size: usize,
     /// Contract address
     pub address: H160,
     /// Caller of the EVM.
@@ -21,10 +23,20 @@ pub struct Contract {
 
 impl Contract {
     pub fn new(input: Bytes, code: Bytes, address: H160, caller: H160, value: U256) -> Self {
-        let jumpdest = ValidJumpAddress::new(code.as_ref());
+        let (jumpdest, padding) = Self::analize(code.as_ref());
+
+        let mut code = code.to_vec();
+        let code_size = code.len();
+        if padding != 0 {
+            code.resize(code.len() + padding + 1, 0);
+        } else {
+            code.resize(code.len() + 1, 0);
+        };
+        let code = code.into();
         Self {
             input,
             code,
+            code_size,
             address,
             caller,
             value,
@@ -32,6 +44,32 @@ impl Contract {
         }
     }
 
+    /// Create a new valid mapping from given code bytes.
+    /// it gives back ValidJumpAddress and size od needed paddings.
+    fn analize(code: &[u8]) -> (ValidJumpAddress, usize) {
+        let mut jumps: Vec<bool> = Vec::with_capacity(code.len());
+        jumps.resize(code.len(), false);
+        let mut is_push_last = false;
+        let mut i = 0;
+        while i < code.len() {
+            let opcode = code[i] as u8;
+            if opcode == opcode::JUMPDEST as u8 {
+                is_push_last = false;
+                jumps[i] = true;
+                i += 1;
+            } else if let Some(v) = OpCode::is_push(opcode) {
+                is_push_last = true;
+                i += v as usize + 1;
+            } else {
+                is_push_last = false;
+                i += 1;
+            }
+        }
+        let padding = if is_push_last { i - code.len() } else { 0 };
+
+        (ValidJumpAddress(jumps), padding)
+    }
+
     pub fn is_valid_jump(&self, possition: usize) -> bool {
         self.jumpdest.is_valid(possition)
     }
@@ -45,22 +83,6 @@ impl Contract {
             call_context.apparent_value,
         )
     }
-
-    pub fn opcode(&self, program_counter: usize) -> Result<OpCode, ExitReason> {
-        let opcode = {
-            if let Some(opcode_byte) = self.code.get(program_counter) {
-                let opcode = OpCode::try_from_u8(*opcode_byte);
-                // if there is no opcode in code or OpCode is invalid, return error.
-                if opcode.is_none() {
-                    return Err(ExitError::OpcodeNotFound.into()); // TODO this not seems right, for invalid opcode
-                }
-                opcode.unwrap()
-            } else {
-                return Err(ExitSucceed::Stopped.into());
-            }
-        };
-        Ok(opcode)
-    }
 }
 
 /// Mapping of valid jump destination from code.
@@ -68,27 +90,6 @@ impl Contract {
 pub struct ValidJumpAddress(Vec<bool>);
 
 impl ValidJumpAddress {
-    /// Create a new valid mapping from given code bytes.
-    pub fn new(code: &[u8]) -> Self {
-        let mut jumps: Vec<bool> = Vec::with_capacity(code.len());
-        jumps.resize(code.len(), false);
-
-        let mut i = 0;
-        while i < code.len() {
-            let opcode = code[i] as u8;
-            if opcode == OpCode::JUMPDEST as u8 {
-                jumps[i] = true;
-                i += 1;
-            } else if let Some(v) = OpCode::is_push(opcode) {
-                i += v as usize + 1;
-            } else {
-                i += 1;
-            }
-        }
-
-        Self(jumps)
-    }
-
     /// Get the length of the valid mapping. This is the same as the
     /// code bytes.
     #[inline]
@@ -112,3 +113,19 @@ impl ValidJumpAddress {
         self.0[position]
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn analize_padding_dummy() {
+        let (_, padding) = Contract::analize(&[opcode::CODESIZE, opcode::PUSH1, 0x00]);
+        assert_eq!(padding, 0, "Padding should be zero");
+    }
+    #[test]
+    fn analize_padding_two_missing() {
+        let (_, padding) = Contract::analize(&[opcode::CODESIZE, opcode::PUSH3, 0x00]);
+        assert_eq!(padding, 2, "Padding should be zero");
+    }
+}
