@@ -1,10 +1,12 @@
-use crate::{alloc::vec::Vec, opcode::eval, ExitError};
+use crate::{
+    instructions::{eval, Return},
+    return_ok, return_revert,
+};
 use bytes::Bytes;
-use core::{cmp::max, ops::Range};
-use primitive_types::U256;
+use core::ops::Range;
 
 use super::{contract::Contract, memory::Memory, stack::Stack};
-use crate::{error::ExitReason, opcode::Control, spec::Spec, Handler};
+use crate::{spec::Spec, Handler};
 
 pub const STACK_LIMIT: u64 = 1024;
 pub const CALL_STACK_LIMIT: u64 = 1024;
@@ -15,7 +17,7 @@ pub struct Machine {
     /// Program counter.
     pub program_counter: usize,
     /// Return value.
-    pub return_range: Range<U256>,
+    pub return_range: Range<usize>,
     /// Memory.
     pub memory: Memory,
     /// Stack.
@@ -26,6 +28,7 @@ pub struct Machine {
     pub gas: Gas,
     /// used only for inspector.
     pub call_depth: u64,
+    //pub times: [(std::time::Duration, usize); 256],
 }
 
 #[derive(Clone, Copy, Debug)]
@@ -34,6 +37,7 @@ pub struct Gas {
     used: u64,
     memory: u64,
     refunded: i64,
+    all_used_gas: u64,
 }
 impl Gas {
     pub fn new(limit: u64) -> Self {
@@ -42,26 +46,23 @@ impl Gas {
             used: 0,
             memory: 0,
             refunded: 0,
+            all_used_gas: 0,
         }
     }
 
-    pub fn reimburse_unspend(&mut self, exit: &ExitReason, other: Gas) {
-        match exit {
-            ExitReason::Succeed(_) => {
+    pub fn reimburse_unspend(&mut self, exit: &Return, other: Gas) {
+        match *exit {
+            return_ok!() => {
                 self.erase_cost(other.remaining());
                 self.record_refund(other.refunded());
             }
-            ExitReason::Revert(_) => {
+            return_revert!() => {
                 self.erase_cost(other.remaining());
             }
             _ => {}
         }
     }
 
-    pub fn limit_mut(&mut self) -> &mut u64 {
-        &mut self.limit
-    }
-
     pub fn limit(&self) -> u64 {
         self.limit
     }
@@ -75,15 +76,16 @@ impl Gas {
     }
 
     pub fn spend(&self) -> u64 {
-        self.used + self.memory
+        self.all_used_gas
     }
 
     pub fn remaining(&self) -> u64 {
-        (self.limit - self.used) - self.memory
+        self.limit - self.all_used_gas
     }
 
     pub fn erase_cost(&mut self, returned: u64) {
         self.used -= returned;
+        self.all_used_gas -= returned;
     }
 
     pub fn record_refund(&mut self, refund: i64) {
@@ -93,32 +95,28 @@ impl Gas {
     /// Record an explict cost.
     #[inline(always)]
     pub fn record_cost(&mut self, cost: u64) -> bool {
-        let all_used_gas: u128 = self.used as u128 + self.memory as u128 + cost as u128;
-        if (self.limit as u128) < all_used_gas {
+        let (all_used_gas, overflow) = self.all_used_gas.overflowing_add(cost);
+        if overflow || self.limit < all_used_gas {
             return false;
         }
 
         self.used += cost;
+        self.all_used_gas = all_used_gas;
         true
     }
 
     /// used in memory_resize! macro
-    pub fn record_memory(&mut self, gas_memory: u64) -> bool {
-        let max_memory = max(self.memory, gas_memory);
-        let all_used_gas: u128 = self.used as u128 + gas_memory as u128;
-        if (self.limit as u128) < all_used_gas {
-            return false;
-        }
-        self.memory = max_memory;
-        true
-    }
-
     #[inline(always)]
-    pub fn record_cost_control(&mut self, cost: u64) -> Control {
-        if !self.record_cost(cost) {
-            return Control::Exit(ExitReason::Error(ExitError::OutOfGas));
+    pub fn record_memory(&mut self, gas_memory: u64) -> bool {
+        if gas_memory > self.memory {
+            let (all_used_gas, overflow) = self.used.overflowing_add(gas_memory);
+            if overflow || self.limit < all_used_gas {
+                return false;
+            }
+            self.memory = gas_memory;
+            self.all_used_gas = all_used_gas;
         }
-        Control::Continue
+        true
     }
 
     /// used in gas_refund! macro
@@ -133,11 +131,12 @@ impl Machine {
             program_counter: 0,
             return_range: Range::default(),
             memory: Memory::new(usize::MAX),
-            stack: Stack::new(STACK_LIMIT as usize),
+            stack: Stack::new(),
             return_data_buffer: Bytes::new(),
             contract,
             gas: Gas::new(gas_limit),
             call_depth,
+            //times: [(std::time::Duration::ZERO, 0); 256],
         }
     }
     pub fn contract(&self) -> &Contract {
@@ -152,94 +151,73 @@ impl Machine {
     pub fn stack(&self) -> &Stack {
         &self.stack
     }
-    /// Mutable reference of machine stack.
-    pub fn stack_mut(&mut self) -> &mut Stack {
-        &mut self.stack
-    }
-    /// Reference of machine memory.
-    pub fn memory(&self) -> &Memory {
-        &self.memory
-    }
-    /// Mutable reference of machine memory.
-    pub fn memory_mut(&mut self) -> &mut Memory {
-        &mut self.memory
-    }
+
     /// Return a reference of the program counter.
     pub fn program_counter(&self) -> usize {
         self.program_counter
     }
 
     /// loop steps until we are finished with execution
-    pub fn run<H: Handler, SPEC: Spec>(&mut self, handler: &mut H) -> ExitReason {
+    pub fn run<H: Handler, SPEC: Spec>(&mut self, handler: &mut H) -> Return {
+        //let timer = std::time::Instant::now();
         loop {
-            if let Err(reason) = self.step::<H, SPEC>(handler) {
+            let ret = self.step::<H, SPEC>(handler);
+            if Return::Continue != ret {
                 if H::INSPECT {
-                    handler.inspect().call_return(reason.clone());
+                    handler.inspect().call_return(ret);
                 }
-                return reason;
+                // let elapsed = timer.elapsed();
+                // println!("run took:{:?}", elapsed);
+                // let mut it = self
+                //     .times
+                //     .iter()
+                //     .zip(crate::OPCODE_JUMPMAP.iter())
+                //     .filter(|((time, _), opcode)| opcode.is_some() && !time.is_zero())
+                //     .map(|((dur, num), code)| (code.unwrap(), dur, num, *dur / *num as u32))
+                //     .collect::<Vec<_>>();
+                // it.sort_by(|a, b| a.2.cmp(&b.2));
+                // for i in it {
+                //     println!(
+                //         "code:{:?}   called:{:?}   time:{:?}   avrg:{:?}",
+                //         i.0,
+                //         i.2,
+                //         i.1,
+                //         i.3,
+                //     );
+                // }
+                return ret;
             }
         }
     }
 
-    #[inline]
+    #[inline(always)]
     /// Step the machine, executing one opcode. It then returns.
-    pub fn step<H: Handler, SPEC: Spec>(&mut self, handler: &mut H) -> Result<(), ExitReason> {
+    pub fn step<H: Handler, SPEC: Spec>(&mut self, handler: &mut H) -> Return {
         if H::INSPECT {
             handler.inspect().step(self);
         }
         // extract next opcode from code
-        let _program_counter = self.program_counter;
-        let opcode = self.contract.opcode(self.program_counter)?;
+        let opcode = unsafe { *self.contract.code.get_unchecked(self.program_counter) };
 
         // evaluate opcode/execute instruction
-        let mut eval = eval::<H, SPEC>(self, opcode, self.program_counter, handler);
+        self.program_counter += 1;
+        let eval = eval::<H, SPEC>(self, opcode, handler);
         if H::INSPECT {
-            handler.inspect().eval(&mut eval, self);
+            handler.inspect().eval(eval, self);
         }
-        match eval {
-            Control::Continue => {
-                self.program_counter += 1;
-            }
-            Control::ContinueN(p) => {
-                self.program_counter += p;
-            }
-            Control::Exit(e) => {
-                return Err(e);
-            }
-            Control::Jump(p) => {
-                self.program_counter = p;
-            }
-        }
-
-        Ok(())
+        eval
     }
 
     /// Copy and get the return value of the machine, if any.
     pub fn return_value(&self) -> Bytes {
-        if self.return_range.start > U256::from(usize::MAX) {
-            let mut ret = Vec::new();
-            ret.resize(
-                (self.return_range.end - self.return_range.start).as_usize(),
-                0,
-            );
-            Bytes::from(ret)
-        } else if self.return_range.end > U256::from(usize::MAX) {
-            let mut ret = self
-                .memory
-                .get(
-                    self.return_range.start.as_usize(),
-                    usize::MAX - self.return_range.start.as_usize(),
-                )
-                .to_vec();
-            while ret.len() < (self.return_range.end - self.return_range.start).as_usize() {
-                ret.push(0);
-            }
-            Bytes::from(ret)
+        // if start is usize max it means that our return len is zero and we need to return empty
+        if self.return_range.start == usize::MAX {
+            Bytes::new()
         } else {
-            self.memory.get(
-                self.return_range.start.as_usize(),
-                (self.return_range.end - self.return_range.start).as_usize(),
-            )
+            Bytes::copy_from_slice(self.memory.get_slice(
+                self.return_range.start,
+                self.return_range.end - self.return_range.start,
+            ))
         }
     }
 }
