@@ -1,6 +1,8 @@
-mod contract;
+#[allow(clippy::module_inception)]
 mod machine;
-mod memory;
+
+mod contract;
+pub(crate) mod memory;
 mod stack;
 
 pub use contract::Contract;
