@@ -52,8 +52,8 @@ impl AccountInfo {
     }
 
     pub fn is_empty(&self) -> bool {
-        let code_empty = self.code_hash == KECCAK_EMPTY || self.code_hash == H256::zero();
-        self.balance == U256::zero() && self.nonce == 0 && code_empty
+        let code_empty = self.code_hash == KECCAK_EMPTY || self.code_hash.is_zero();
+        self.balance.is_zero() && self.nonce == 0 && code_empty
     }
 
     pub fn exists(&self) -> bool {
@@ -61,9 +61,10 @@ impl AccountInfo {
     }
 
     pub fn from_balance(balance: U256) -> Self {
-        let mut def = Self::default();
-        def.balance = balance;
-        def
+        AccountInfo {
+            balance,
+            ..Default::default()
+        }
     }
 }
 
@@ -94,7 +95,7 @@ pub enum CreateScheme {
     /// Create scheme of `CREATE2`.
     Create2 {
         /// Salt.
-        salt: H256,
+        salt: U256,
     },
 }
 
@@ -154,12 +155,13 @@ pub struct TxEnv {
     pub data: Bytes,
     pub chain_id: Option<u64>,
     pub nonce: Option<u64>,
-    pub access_list: Vec<(H160, Vec<H256>)>,
+    pub access_list: Vec<(H160, Vec<U256>)>,
 }
 #[derive(Clone, Debug)]
 pub struct CfgEnv {
     pub chain_id: U256,
     pub spec_id: SpecId,
+    pub use_gas: bool,
 }
 
 impl Default for CfgEnv {
@@ -167,6 +169,7 @@ impl Default for CfgEnv {
         CfgEnv {
             chain_id: 1.into(), //mainnet is 1
             spec_id: SpecId::LATEST,
+            use_gas: true,
         }
     }
 }
